/*
Filename: i2c.c
Description: 
I2C Slave Driver for Power Controller of EN3000. This software interfaces with the main CPU in slave mode to control the LEDs and other IOs in Nrodic chip
Author - Suman Manandhar
*/
#include <stdbool.h>
#include <stdint.h>
#include <string.h>
#include "config.h"
#include "i2c.h"
#include "nrf_drv_twis.h"
#include "nrf_gpio.h"
#include "nrf_delay.h"
#include "app_util_platform.h"
#include "sdk_config.h"
#include "nrf_log_ctrl.h"

/**
 * @ingroup eeprom_simulator
 * @defgroup eeprom_simulator_ivars EEPROM simulator internal variables
 *
 * Internal variables required for the module to work.
 * @{
 */

    /**
     * @brief Memory array.
     *
     * Place that will simulate the memory.
     */
    static uint8_t m_memory[I2C_ADDR_SIZE];

    /**
     * @brief Current memory address.
     *
     * Memory pointer for any operation that will be processed on memory array.
     */
    static uint16_t m_addr = 0;

    /**
     * @brief Receive buffer.
     *
     * Receiving buffer must contain the address and EEPROM_SIM_SEQ_WRITE_MAX_BYTES bytes of data.
     */
    static uint8_t m_rxbuff[I2C_ADDR_ADDRESS_LEN_BYTES + I2C_ADDR_SEQ_WRITE_MAX_BYTES];

    /**
     * @brief Internal error flag.
     *
     * This flag is set if any communication error is detected.
     * It can be cleared by calling the @ref eeprom_simulator_error_get function.
     */
    static bool m_error_flag;

    /**
     * @brief TWIS instance.
     *
     * TWIS driver instance used by this EEPROM simulator.
     */
    static const nrf_drv_twis_t m_twis = NRF_DRV_TWIS_INSTANCE(I2C_ADDR_TWIS_INST);

    /**
     * @brief I2c Slave configuration.
     *
     * When the I2C is initialized, the configuration is stored here.
     */
    static i2c_config_t m_i2c_config;


/** @} */

    /**
     * @brief Set the current address pointer.
     *
     * Sets address for the next operation on the memory array.
     * @param addr Address to set.
     */
    static void i2c_setAddr(uint16_t addr)
    {
        m_addr = addr;
    }

    static void do_reset(void * p_context)
    {
      UNUSED_PARAMETER(p_context);

      NRF_LOG_FINAL_FLUSH();

      nrf_delay_ms(NRF_RESET_DELAY_MS);

      NVIC_SystemReset();
    }

unsigned int led_value = 0;
unsigned int get_leds_value(void)
{
	return led_value;
}


void mod1_gled_on()
{
	nrf_gpio_pin_clear(40);	
	led_value &= ~0x01;
}
void mod1_gled_off()
{
	nrf_gpio_pin_set(40);	
	led_value |= 0x01;
}

void mod1_rled_on()
{
	nrf_gpio_pin_clear(41);	
	led_value &= ~0x02;
}
void mod1_rled_off()
{
	nrf_gpio_pin_set(41);	
	led_value |= 0x02;
}
void mod1_bled_on()
{
	nrf_gpio_pin_clear(5);	
	led_value &= ~0x04;
}
void mod1_bled_off()
{
	nrf_gpio_pin_set(5);	
	led_value |= 0x04;
}


void mod2_gled_on()
{
	nrf_gpio_pin_clear(27);	
	led_value &= ~0x08;
}
void mod2_gled_off()
{
	nrf_gpio_pin_set(27);	
	led_value |= 0x08;
}

void mod2_rled_on()
{
	nrf_gpio_pin_clear(10);	
	led_value &= ~0x10;

}
void mod2_rled_off()
{
	nrf_gpio_pin_set(10);	
	led_value |= 0x10;
}
void mod2_bled_on()
{
	nrf_gpio_pin_clear(26);	
	led_value &= ~0x20;
}
void mod2_bled_off()
{
	nrf_gpio_pin_set(26);	
	led_value |= 0x20;
}

void sys_gled_on()
{
	nrf_gpio_pin_clear(11);	
	led_value &= ~0x40;
}
void sys_gled_off()
{
	nrf_gpio_pin_set(11);	
	led_value |= 0x40;
}

void sys_rled_on()
{
	nrf_gpio_pin_clear(12);	
	led_value &= ~0x80;

}
void sys_rled_off()
{
	nrf_gpio_pin_clear(12);	
	led_value |= 0x80;
}

void sys_bled_on()
{
	nrf_gpio_pin_clear(7);	
	led_value &= ~0x100;

}
void sys_bled_off()
{
	nrf_gpio_pin_clear(7);	
	led_value |= 0x100;
}


void wifi_gled_on()
{
	nrf_gpio_pin_clear(0);	
	led_value &= ~0x200;
}
void wifi_gled_off()
{
	nrf_gpio_pin_set(0);	
	led_value |= 0x200;
}

void wifi_rled_on()
{
	led_value &= ~0x400;

}
void wifi_rled_off()
{
	led_value |= 0x400;
}

void wifi_bled_on()
{
	nrf_gpio_pin_clear(1);	
	led_value &= ~0x800;

}
void wifi_bled_off()
{
	nrf_gpio_pin_set(1);	
	led_value |= 0x800;
}

void set_leds_state(uint32_t led_data, int addr)
{
        if (addr == 0) {
            if (led_data & 0x01)
		mod1_gled_off();
            else	
		mod1_gled_on();
		
            if (led_data & 0x02)
		mod1_rled_off();
            else
		mod1_rled_on();	
            
            if (led_data & 0x04)
		mod2_gled_off();
            else
		mod2_gled_on();
	
            if (led_data & 0x08)
		mod2_rled_off();
            else
		mod2_rled_on();

            if (led_data & 0x10)
		sys_gled_off();
            else
		sys_gled_on();
	
            if (led_data & 0x20)
		sys_rled_off();
            else
		sys_rled_on();
            if (led_data & 0x40)
		wifi_gled_off();
            else
		wifi_gled_on();
	
            if (led_data & 0x80)
		wifi_rled_off();
            else
		wifi_rled_on();
        } else {
            if (led_data & 0x01)
		mod1_bled_off();
            else	
		mod1_bled_on();
		
            if (led_data & 0x02)
		mod2_bled_off();
            else
		mod2_bled_on();	
            
            if (led_data & 0x04)
		sys_bled_off();
            else
		sys_bled_on();
	
            if (led_data & 0x08)
		wifi_bled_off();
            else
		wifi_bled_on();            
        }
}

    /**
     * @brief Perform a write operation on the memory array.
     *
     * Write a single byte into the memory array using @ref m_addr as a write pointer.
     * @param data Data to be written.
     */
    static void i2c_write(uint8_t data)
    {
        if (m_addr >= sizeof(m_memory))
            m_addr = 0;
        m_memory[m_addr] = data;
        switch(m_addr)
        {
	default:
            break;
	case 0x20:
		do_reset(NULL);
		break;	
	case 0x1a:
		//Bluetooth
		break;		
	case 4:
		set_leds_state(data, 0);
		break;	
	case 2:
		break;
	case 5:
		if (data & 0x08)
			nrf_gpio_pin_clear(20);
		else	
			nrf_gpio_pin_set(20);
                if (data & 0x04)
			nrf_gpio_pin_clear(19);
                else	
			nrf_gpio_pin_set(19);
                break;
						
	case 6:		
		break;
			
	case 7:
		if (data & 0x08)
			nrf_gpio_pin_clear(33);
                else	
			nrf_gpio_pin_set(33);
                if (data & 0x04)
			nrf_gpio_pin_clear(34);
		else	
                        nrf_gpio_pin_set(34);
		break;

	case 0x0C:
	case 0x0D:
	case 0x0E:
		break;		
	}
	m_addr++;
    }

    /**
     * @brief Start after the WRITE command.
     *
     * Function sets pointers for TWIS to receive data.
     * WRITE command does not write directly to memory array.
     * Temporary receive buffer is used.
     * @sa m_rxbuff
     */
    static void i2c_writeBegin(void)
    {
        (void)nrf_drv_twis_rx_prepare(&m_twis, m_rxbuff, sizeof(m_rxbuff));
    }

    /**
     * @brief Finalize the WRITE command.
     *
     * Call this function when the write command is finished.
     * It sets a memory pointer and writes all received data to the memory array.
     */
    static void i2c_writeEnd(size_t cnt)
    {
        if (cnt > (I2C_ADDR_ADDRESS_LEN_BYTES - 1))
        {
            size_t n;
#if (I2C_ADDR_ADDRESS_LEN_BYTES == 1)
            i2c_setAddr(m_rxbuff[0]);
#endif

#if (I2C_ADDR_ADDRESS_LEN_BYTES == 2)
            uint16_t rxbuff;

            if (TWI_ADDRESS_CONFIG == LITTLE_ENDIAN)
            {
                rxbuff = ((m_rxbuff[1] << 8 ) | (m_rxbuff[0]));
            }
            else
            {
                rxbuff = ((m_rxbuff[0] << 8 ) | (m_rxbuff[1]));
            }

            i2c_setAddr(rxbuff);
#endif



            for (n=I2C_ADDR_ADDRESS_LEN_BYTES; n<cnt; ++n)
            {
                i2c_write(m_rxbuff[n]);
            }
        }
    }

    
    /**
     * @brief Start after the READ command.
     *
     * Function sets pointers for TWIS to transmit data from the current address to the end of memory.
     */
    static void i2c_readBegin(void)
    {
        uint32_t i, read_len = I2C_ADDR_SEQ_READ_MAX_BYTES;
        uint8_t ch;
        volatile uint16_t temp_addr = m_addr;

        if (m_addr + I2C_ADDR_SEQ_READ_MAX_BYTES > sizeof(m_memory))
        {
            read_len = I2C_ADDR_SIZE - m_addr;
            temp_addr = 0;
        }
        for (i = 0; i<I2C_ADDR_SEQ_READ_MAX_BYTES; i++) {
            ch = 0;
            switch(m_addr+i)
            {
            default:	
            case 0x1a:		
            case 0:	
                  ch = 0x02;//version
                  break;
					
            case 2:
                  ch = 0x02;//version
                  break;
            case 3:
                  ch = 0x80;	
                  break;
						
            case 4:
                  //leds
                  ch = led_value;//led_value);
                  break;
						
            case 5:
                  /* slot 0 power,reset,wdisable */
                  ch = nrf_gpio_pin_read(20)<<3;
                  ch |= nrf_gpio_pin_read(19) << 2;	
                  ch |= 0 << 1;
                  break;
						
            case 7:	
                  /* slot 1 power,reset,wdisable */
                  ch = nrf_gpio_pin_read(33);
                  ch |= nrf_gpio_pin_read(34) << 2;	
                  ch |= 0 << 1;
                  break;
            case 6:	
                  /* slot 1 power,reset,wdisable */
                  ch = led_value >> 8;
                  break;
            case 0x12:
                  //ch = sampled_adc_values[1]&0xff;
                  break;	
						
            case 0x13:
                  //data = (sampled_adc_values[1] >> 8) & 0xFF;
                  break;
						
            case 0x0C:
            case 0x0D:
            case 0x0E:
                  ch =  0xff;
                  break;
            }
            m_memory[m_addr+i] = ch;
        }
	(void) nrf_drv_twis_tx_prepare(&m_twis, m_memory + m_addr, read_len);
        m_addr += i;

    }

    /**
     * @brief Finalize the READ command.
     *
     * Call this function when the read command is finished.
     * It adjusts the current m_addr pointer.
     * @param cnt Number of bytes read.
     */
    static void i2c_readEnd(size_t cnt)
    {
        m_addr += cnt;
    }


    /**
     * @brief Event processing.
     *
     *
     */
    static void twis_event_handler(nrf_drv_twis_evt_t const * const p_event)
    {
        switch (p_event->type)
        {
        case TWIS_EVT_READ_REQ:
            if (p_event->data.buf_req)
            {
                i2c_readBegin();
            }
            break;
        case TWIS_EVT_READ_DONE:
            i2c_readEnd(p_event->data.tx_amount);
            break;
        case TWIS_EVT_WRITE_REQ:
            if (p_event->data.buf_req)
            {
                i2c_writeBegin();
            }
            break;
        case TWIS_EVT_WRITE_DONE:
            i2c_writeEnd(p_event->data.rx_amount);
            break;

        case TWIS_EVT_READ_ERROR:
        case TWIS_EVT_WRITE_ERROR:
        case TWIS_EVT_GENERAL_ERROR:
            m_error_flag = true;
            break;
        default:
            break;
        }
    }

/** @} */



ret_code_t i2c_slave_init(void)
{
    ret_code_t ret;

    m_i2c_config.slave_addr                = I2C_ADDR;
    m_i2c_config.scl_s                     = I2C_ADDR_SCL_S;
    m_i2c_config.sda_s                     = I2C_ADDR_SDA_S;
    
    m_addr = 0;

    const nrf_drv_twis_config_t config =
    {
        .addr               = {m_i2c_config.slave_addr, 0},
        .scl                = m_i2c_config.scl_s,
        .scl_pull           = NRF_GPIO_PIN_PULLUP,
        .sda                = m_i2c_config.sda_s,
        .sda_pull           = NRF_GPIO_PIN_PULLUP,
        .interrupt_priority = APP_IRQ_PRIORITY_HIGH
    };

    /* Init TWIS */
    do
    {
        ret = nrf_drv_twis_init(&m_twis, &config, twis_event_handler);
        if (NRF_SUCCESS != ret)
        {
            break;
        }

        nrf_drv_twis_enable(&m_twis);
    }while (0);
    return ret;
}

bool i2c_error_check(void)
{
    return m_error_flag;
}

uint32_t i2c_error_get_and_clear(void)
{
    uint32_t ret = nrf_drv_twis_error_get_and_clear(&m_twis);
    m_error_flag = false;
    return ret;
}
